Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports Microsoft.AnalysisServices
Imports System.IO
Imports System.Xml

Namespace Tests
	Public Class XMLAtoCube
		Private cubeConfigBaseDir As String

		Public Sub New(ByVal cubeConfigBaseDir As String)
			Me.cubeConfigBaseDir = cubeConfigBaseDir
		End Sub

'''         Connect to an Analysis server using the given connection string
'''         * connectionstring - Anaylsis server URL
'''         
		Private Function ConnectToServer(ByVal connectionstring As String) As Server
            Dim server As Server
			server.Connect(connectionstring)
			Return server
		End Function

'''         Drops the a database on the connected Analysis server 
'''         * server - The connected Analysis server
'''         * databaseName - The name of the unit-test version of the database
'''         
		Private Sub DropDatabase(ByVal server As Server, ByVal databaseName As String)
			Dim db As Database = Nothing
			If server.Connected Then
				' Find the Database instance given its name
				db = server.Databases.FindByName(databaseName)
				If db IsNot Nothing Then
					' If the database already exists that its dropped
					db.Drop()
				End If
			End If
		End Sub

'''         Retrieve the datbase instance on the connected Analysis serer given the database's name
'''         * server - The connected Analysis server
'''         * databaseName - The name of the unit-test version of the database
'''         
		Private Function GetDatabase(ByVal server As Server, ByVal databaseName As String) As Database
			Dim db As Database = Nothing
			If server.Connected Then
				db = server.Databases.FindByName(databaseName)
			End If
			Return db
		End Function

'''         Creates the unit-test version of the Cube given the production version's XMLA definition.  The unit test
'''         * version is created on the specified target Analysis server and database.
'''         * baseDir - The base directory where the XMLA definition is stored.
'''         * sourceServerName - The name of the server where the production version of the Cube resides.
'''         * sourceDatabaseName - The name of the database where the prodution version of the Cube resides.
'''         * targetDatabaseName - The name of the unit-test version of the Cube database.
'''         * dataSourceProviderDefinition - The connection string URL pointing to the test version of the source dimensions and fact tables.
'''         * targetServer - The name of the unit-test Analysis server.
'''         
		Private Sub AddCubeToDatabase(ByVal baseDir As String, ByVal sourceServerName As String, ByVal sourceDatabaseName As String, ByVal targetDatabaseName As String, ByVal dataSourceProviderDefinition As String, ByVal targetServer As Server)
			' Get the XMLA Cube definition file
			Dim cubeDbs() As String = Directory.GetFiles(baseDir & "\" & sourceServerName & "\" & sourceDatabaseName & "\", "*.xmla")
			' Read the XMLA definition into memory and store it within a Database instance                        
			Dim reader As New XmlTextReader(cubeDbs(0))
            Dim database As Database
			Microsoft.AnalysisServices.Utils.Deserialize(reader, database)
			' Set the database name to targetDatabaseName
			database.Name = targetDatabaseName
			database.ID = targetDatabaseName
			' Add the unit-test database to the target Analysis server
			targetServer.Databases.Add(database)
			database.Update()
			' Set the Cube's dimension/fact table datasource to the unit-test version.
			For Each ds As DataSource In database.DataSources
				ds.ConnectionString = dataSourceProviderDefinition
				ds.Update()
			Next ds

			' Update the data source views
			For Each dsv As DataSourceView In database.DataSourceViews
				dsv.Update()
			Next dsv

			' Update the dimensions
            For Each dimen As Microsoft.AnalysisServices.Dimension In database.Dimensions
                dimen.Update()
            Next dimen

			' Update the cube
			For Each cube As Cube In database.Cubes
				cube.Update(UpdateOptions.ExpandFull)
			Next cube
		End Sub

		''' Process all the dimensions within the Database 
		Private Sub ProcessDimensions(ByVal db As Database)
            For Each dimen As Microsoft.AnalysisServices.Dimension In db.Dimensions
                ' Process the dimension and all objects contained within.
                dimen.Process(ProcessType.ProcessFull)
            Next dimen
		End Sub

		''' Process all the Cube(s) within the database 
		Private Sub ProcessCube(ByVal db As Database)
			For Each cube As Cube In db.Cubes
		' Process the cube and all objects contained within.
				cube.Process(ProcessType.ProcessFull)
			Next cube
		End Sub

'''        
'''         * Creates a new Cube on a given server using a given database based on the XMLA of an existing cube.
'''         * sourceServerName - The name of the Analysis server where the current cube resides
'''         * targetServerName - Th name of the Analysis server where the unit-test version of the cube will be placed
'''         * sourceDatabaseName - The name of the database where the current cube resides
'''         * targetDatabaseBame - The name of the database where the unit-test version of the cube will be placed
'''         * dataSourceProviderDefinition - The connection string URL where the test version of the dimension and fact tables reside
'''         *
		Public Sub CreateCubeFromXMLA(ByVal sourceServerName As String, ByVal targetServerName As String, ByVal sourceDatabaseName As String, ByVal targetDatabaseName As String, ByVal dataSourceProviderDefinition As String)
			' Connect to the unit-test Analysis server
			Dim server As Server = ConnectToServer(targetServerName)
			' Drop the target database if it already exists
			DropDatabase(server, targetDatabaseName)

			' Create the unit-test version of the cube on the given target server and database
			AddCubeToDatabase(cubeConfigBaseDir, sourceServerName, sourceDatabaseName, targetDatabaseName, dataSourceProviderDefinition, server)
			server.Refresh()
			Dim db As Database = GetDatabase(server, targetDatabaseName)
			' Process the dimensions of the new cube
			ProcessDimensions(db)
			' Process the new cube
			ProcessCube(db)
		End Sub
	End Class
End Namespace
